# Cursor movement
HOME = "\x1b[H"  # ]
UP = "\x1b[A"  # ]
DOWN = "\x1b[B"  # ]
RIGHT = "\x1b[C"  # ]
LEFT = "\x1b[D"  # ]
START = "\r"

# Screen clearing
CLEAR_SCREEN_END = "\x1b[0J"  # ]
CLEAR_SCREEN_START = "\x1b[1J"  # ]
CLEAR_SCREEN = "\x1b[2J"  # ]
CLEAR_LINE_END = "\x1b[0K"  # ]
CLEAR_LINE_START = "\x1b[1K"  # ]
CLEAR_LINE = "\x1b[2K"  # ]

# Text styles
BOLD = "\x1b[1m"  # ]
DIM = "\x1b[2m"  # ]
ITALIC = "\x1b[3m"  # ]
UNDERLINE = "\x1b[4m"  # ]
BLINK = "\x1b[5m"  # ]
REVERSE = "\x1b[7m"  # ]
HIDDEN = "\x1b[8m"  # ]
STRIKE = "\x1b[9m"  # ]

# Text colors
RESET = "\x1b[0m"  # ]
BLACK = "\x1b[30m"  # ]
RED = "\x1b[31m"  # ]
GREEN = "\x1b[32m"  # ]
YELLOW = "\x1b[33m"  # ]
BLUE = "\x1b[34m"  # ]
MAGENTA = "\x1b[35m"  # ]
CYAN = "\x1b[36m"  # ]
WHITE = "\x1b[37m"  # ]
BLACK_BRIGHT = "\x1b[90m"  # ]
RED_BRIGHT = "\x1b[91m"  # ]
GREEN_BRIGHT = "\x1b[92m"  # ]
YELLOW_BRIGHT = "\x1b[93m"  # ]
BLUE_BRIGHT = "\x1b[94m"  # ]
MAGENTA_BRIGHT = "\x1b[95m"  # ]
CYAN_BRIGHT = "\x1b[96m"  # ]
WHITE_BRIGHT = "\x1b[97m"  # ]

# Background colors
BG_BLACK = "\x1b[40m"  # ]
BG_RED = "\x1b[41m"  # ]
BG_GREEN = "\x1b[42m"  # ]
BG_YELLOW = "\x1b[43m"  # ]
BG_BLUE = "\x1b[44m"  # ]
BG_MAGENTA = "\x1b[45m"  # ]
BG_CYAN = "\x1b[46m"  # ]
BG_WHITE = "\x1b[47m"  # ]
BG_BLACK_BRIGHT = "\x1b[100m"  # ]
BG_RED_BRIGHT = "\x1b[101m"  # ]
BG_GREEN_BRIGHT = "\x1b[102m"  # ]
BG_YELLOW_BRIGHT = "\x1b[103m"  # ]
BG_BLUE_BRIGHT = "\x1b[104m"  # ]
BG_MAGENTA_BRIGHT = "\x1b[105m"  # ]
BG_CYAN_BRIGHT = "\x1b[106m"  # ]
BG_WHITE_BRIGHT = "\x1b[107m"  # ]
